// 3D vector class
#ifndef Vec_hpp
#define Vec_hpp

#include <cmath>

// 3D vector, access as vec.x, vec.y, and vec.z
struct Vec3 {
	float x, y, z;
};

//////////////////////////////////////////////////////////////////////
// factory for Vec3
inline Vec3 vec3(float x0, float y0, float z0) {
	Vec3 v = {x0, y0, z0};
	return v;
}

//////////////////////////////////////////////////////////////////////
// vector add and subtract v0+v1 or v0-v1
inline Vec3 operator+(const Vec3 &v1, const Vec3 &v2) {
	return vec3(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z);
}

inline Vec3 operator-(const Vec3 &v1, const Vec3 &v2) {
	return vec3(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z);
}

//////////////////////////////////////////////////
// operations between a scalar and vector: +, -, *
inline Vec3 operator-(const Vec3 &v1) {
	return vec3(-v1.x, -v1.y, -v1.z);
}

inline Vec3 operator+(float s, const Vec3 &v1) {
	return vec3(s + v1.x, s + v1.y, s + v1.z);
}

inline Vec3 operator-(float s, const Vec3 &v1) {
	return vec3(s - v1.x, s - v1.y, s - v1.z);
}

inline Vec3 operator*(float s, const Vec3 &v1) {
	return vec3(s * v1.x, s * v1.y, s * v1.z);
}


//////////////////////////////////////////////////
// operations between a vector and a scalar: +, -, *, /
inline Vec3 operator+(const Vec3 &v1, float s) {
	return vec3(v1.x + s, v1.y + s, v1.z + s);
}

inline Vec3 operator-(const Vec3 &v1, float s) {
	return vec3(v1.x - s, v1.y - s, v1.z - s);
}

inline Vec3 operator*(const Vec3 &v1, float s) {
	return vec3(v1.x * s, v1.y * s, v1.z * s);
}

inline Vec3 operator/(const Vec3 &v1, float s) {
	return vec3(v1.x / s, v1.y / s, v1.z / s);
}



//////////////////////////////////////////////////////////////////////
// other linear algebra operations

// vector dot product, dot(v1, v2)
inline float dot(const Vec3 &v1, const Vec3 &v2) {
	return v1.x * v2.x  +  v1.y * v2.y  +  v1.z * v2.z;
}

// vector length, length(v1)
inline float length(const Vec3 &v1) {
	return sqrtf(dot(v1, v1));
}

// normalized vector, normalize(v1)
inline Vec3 normalize(const Vec3 &v1) {
	return v1 / length(v1);
}

// cross product, v1^v2
inline Vec3 operator^(const Vec3 &v1, const Vec3 &v2) {
	return vec3(v1.y*v2.z - v1.z*v2.y,
				v1.z*v2.x - v1.x*v2.z,
				v1.x*v2.y - v1.y*v2.x);
}

#endif
